@@ -1,4 +1,4 @@
-/*	$OpenBSD: ntp.h,v 1.9 2004/10/13 13:35:19 henning Exp $ */
+/*	$OpenBSD: ntp.h,v 1.11 2004/12/13 12:22:52 dtucker Exp $ */
 
 /*
  * Copyright (c) 2004 Henning Brauer <henning@openbsd.org>
@@ -40,13 +40,13 @@
  *   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
 struct l_fixedpt {
-	u_int32_t int_part;
-	u_int32_t fraction;
+	u_int32_t int_partl;
+	u_int32_t fractionl;
 };
 
 struct s_fixedpt {
-	u_int16_t int_part;
-	u_int16_t fraction;
+	u_int16_t int_parts;
+	u_int16_t fractions;
 };
 
 /* RFC Section 4
@@ -145,5 +145,6 @@ struct ntp_query {
 #define	JAN_1970	2208988800UL	/* 1970 - 1900 in seconds */
 
 #define	NTP_VERSION	4
+#define	NTP_MAXSTRATUM	15
 
 #endif	/* _NTP_H_ */
