@@ -1,4 +1,4 @@
-/*	$OpenBSD: server.c,v 1.21 2005/01/28 12:01:32 dtucker Exp $ */
+/*	$OpenBSD: server.c,v 1.28 2006/01/19 11:20:23 dtucker Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -34,6 +34,8 @@ setup_listeners(struct servent *se, struct ntpd_conf *conf, u_int *cnt)
 	struct listen_addr	*la;
 	struct ifaddrs		*ifa, *ifap;
 	struct sockaddr		*sa;
+	u_int8_t		*a6;
+	size_t			 sa6len = sizeof(struct in6_addr);
 	u_int			 new_cnt = 0;
 	int			 tos = IPTOS_LOWDELAY;
 
@@ -44,9 +46,24 @@ setup_listeners(struct servent *se, struct ntpd_conf *conf, u_int *cnt)
 		for (ifap = ifa; ifap != NULL; ifap = ifap->ifa_next) {
 			sa = ifap->ifa_addr;
 
-			if (sa->sa_family != AF_INET &&
-			    sa->sa_family != AF_INET6)
+			if (sa == NULL ||
+			    (sa->sa_family != AF_INET &&
+			    sa->sa_family != AF_INET6))
 				continue;
+			if (SA_LEN(sa) == 0)
+				continue;
+
+			if (sa->sa_family == AF_INET &&
+			    ((struct sockaddr_in *)sa)->sin_addr.s_addr ==
+			    INADDR_ANY)
+				continue;
+
+			if (sa->sa_family == AF_INET6) {
+				a6 = ((struct sockaddr_in6 *)sa)->
+				    sin6_addr.s6_addr;
+				if (memcmp(a6, &in6addr_any, sa6len) == 0)
+					continue;
+			}
 
 			if ((la = calloc(1, sizeof(struct listen_addr))) ==
 			    NULL)
@@ -122,13 +139,17 @@ server_dispatch(int fd, struct ntpd_conf *conf)
 
 	rectime = gettime();
 
-	if (ntp_getmsg(buf, size, &query) == -1)
+	if (ntp_getmsg((struct sockaddr *)&fsa, buf, size, &query) == -1)
 		return (0);
 
 	version = (query.status & VERSIONMASK) >> 3;
 
 	bzero(&reply, sizeof(reply));
-	reply.status = conf->status.leap | (query.status & VERSIONMASK);
+	if (conf->status.synced)
+		reply.status = conf->status.leap;
+	else
+		reply.status = LI_ALARM;
+	reply.status |= (query.status & VERSIONMASK);
 	if ((query.status & MODEMASK) == MODE_CLIENT)
 		reply.status |= MODE_SERVER;
 	else
@@ -144,7 +165,7 @@ server_dispatch(int fd, struct ntpd_conf *conf)
 	reply.rootdelay = d_to_sfp(conf->status.rootdelay);
 
 	if (version > 3)
-		reply.refid = reply.xmttime.fractionl;
+		reply.refid = conf->status.refid4;
 	else
 		reply.refid = conf->status.refid;
 
